"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPackageUrl = exports.ParseStatus = void 0;
const package_json_1 = __importDefault(require("package-json"));
const validate_npm_package_name_1 = __importDefault(require("validate-npm-package-name"));
const utils_1 = require("./utils");
const logger = (0, utils_1.getLogger)('RepositoryParser');
var ParseStatus;
(function (ParseStatus) {
    ParseStatus["INVALID_PACKAGE_NAME"] = "INVALID_NAME";
    ParseStatus["INVALID_URL"] = "INVALID_URL";
    ParseStatus["NO_URL_FOUND"] = "NO_URL_FOUND";
    ParseStatus["PACKAGE_NOT_FOUND"] = "PACKAGE_NOT_FOUND";
    ParseStatus["SERVER_ERROR"] = "SERVER_ERROR";
    ParseStatus["SUCCESS"] = "SUCCESS";
    ParseStatus["VERSION_NOT_FOUND"] = "VERSION_NOT_FOUND";
})(ParseStatus = exports.ParseStatus || (exports.ParseStatus = {}));
const knownSSLHosts = ['bitbucket.org', 'github.com', 'gitlab.com', 'sourceforge.net'];
function parseRepositoryEntry(repository) {
    return typeof repository === 'string' ? repository : repository.url || null;
}
function cleanUrl(url) {
    url = url.replace(/\.git$/, '').replace(/^.*:\/\//, 'http://');
    const parsedURL = (0, utils_1.validateUrl)(url);
    if (parsedURL) {
        parsedURL.hash = '';
        parsedURL.password = '';
        parsedURL.protocol = knownSSLHosts.includes(parsedURL.hostname) ? 'https:' : 'http:';
        parsedURL.search = '';
        parsedURL.username = '';
        return parsedURL.href;
    }
    return null;
}
function getPackageUrl(rawPackageName, version = 'latest') {
    return __awaiter(this, void 0, void 0, function* () {
        let packageInfo;
        let foundUrl = null;
        let parsedRepository = null;
        const validateResult = (0, validate_npm_package_name_1.default)(rawPackageName);
        if (!validateResult.validForNewPackages) {
            logger.info(`Invalid package name: "${rawPackageName}"`, validateResult);
            return { status: ParseStatus.INVALID_PACKAGE_NAME };
        }
        try {
            packageInfo = yield (0, package_json_1.default)(rawPackageName, { fullMetadata: true, version });
        }
        catch (error) {
            if (error instanceof package_json_1.default.VersionNotFoundError) {
                logger.info(`Version "${version}" not found for package "${rawPackageName}".`);
                return { status: ParseStatus.VERSION_NOT_FOUND };
            }
            if (error instanceof package_json_1.default.PackageNotFoundError) {
                logger.info(`Package "${rawPackageName}" not found.`);
                return { status: ParseStatus.PACKAGE_NOT_FOUND };
            }
            logger.error(error);
            return { status: ParseStatus.SERVER_ERROR };
        }
        if (packageInfo.repository) {
            parsedRepository = parseRepositoryEntry(packageInfo.repository);
        }
        if (parsedRepository) {
            logger.info(`Found repository "${parsedRepository}" for package "${rawPackageName}" (version "${version}").`);
            foundUrl = parsedRepository;
        }
        else if (typeof packageInfo.homepage === 'string') {
            logger.info(`Found homepage "${packageInfo.homepage}" for package "${rawPackageName}" (version "${version}").`);
            foundUrl = packageInfo.homepage;
        }
        else if (typeof packageInfo.url === 'string') {
            logger.info(`Found URL "${packageInfo.url}" for package "${rawPackageName}" (version "${version}").`);
            foundUrl = packageInfo.url;
        }
        if (!foundUrl) {
            logger.info(`No source URL found in package "${rawPackageName}".`);
            return { status: ParseStatus.NO_URL_FOUND };
        }
        foundUrl = cleanUrl(foundUrl);
        if (!foundUrl) {
            logger.info(`Invalid URL "${foundUrl}" for package "${rawPackageName}".`);
            return { status: ParseStatus.INVALID_URL };
        }
        return {
            status: ParseStatus.SUCCESS,
            url: foundUrl,
        };
    });
}
exports.getPackageUrl = getPackageUrl;
//# sourceMappingURL=RepositoryParser.js.map