"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Server = void 0;
const express_1 = __importDefault(require("express"));
const nocache_1 = __importDefault(require("nocache"));
const helmet_1 = __importDefault(require("helmet"));
const routes_1 = require("./routes/");
const utils_1 = require("./utils");
const logger = (0, utils_1.getLogger)('Server');
class Server {
    constructor(config) {
        this.config = config;
        this.app = (0, express_1.default)();
        this.init();
    }
    init() {
        // The order is important here, please don't sort!
        this.initSecurityHeaders();
        this.app.use((0, routes_1.healthCheckRoute)());
        this.app.use((0, routes_1.infoRoute)());
        this.app.use((0, routes_1.mainRoute)());
        (0, routes_1.initSwaggerRoute)(this.app, this.config);
        this.app.use((0, routes_1.packagesRoute)());
        this.app.use((0, routes_1.notFoundRoute)());
        this.app.use((0, routes_1.internalErrorRoute)());
    }
    initCaching() {
        if (this.config.DEVELOPMENT) {
            this.app.use((0, nocache_1.default)());
        }
        else {
            this.app.use((_, res, next) => {
                const milliSeconds = 1000;
                res.header('Cache-Control', `public, max-age=${this.config.CACHE_DURATION_SECONDS}`);
                res.header('Expires', new Date(Date.now() + this.config.CACHE_DURATION_SECONDS * milliSeconds).toUTCString());
                next();
            });
        }
    }
    initSecurityHeaders() {
        this.app.disable('x-powered-by');
        this.app.use((0, helmet_1.default)({
            frameguard: { action: 'deny' },
        }));
        this.app.use(helmet_1.default.noSniff());
        this.app.use(helmet_1.default.xssFilter());
    }
    start() {
        return new Promise((resolve, reject) => {
            if (this.server) {
                reject(new Error('Server is already running.'));
            }
            else if (this.config.PORT_HTTP) {
                this.server = this.app.listen(this.config.PORT_HTTP, () => {
                    logger.info(`Server is running on port ${this.config.PORT_HTTP}.`);
                    resolve(this.config.PORT_HTTP);
                });
            }
            else {
                reject(new Error('Server port not specified.'));
            }
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.server) {
                this.server.close();
                this.server = undefined;
            }
            else {
                throw new Error('Server is not running.');
            }
        });
    }
}
exports.Server = Server;
//# sourceMappingURL=Server.js.map